library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Mux4x4_1x4_VHDL is
    Port ( A,B,C,D :        in  std_logic_vector(3 downto 0);
           Dp3,Dp2,Dp1,Dp0 : in  std_logic;
			  S1,S0 :           in  std_logic;
           Y :               out std_logic_vector(3 downto 0);
           Dp :              out std_logic);
end Mux4x4_1x4_VHDL;

-----------------------------------------------------------------
-- The following concurrent code implements two multiplexers
-- One 4x4-bit 
-- One 1x4-bit 
-----------------------------------------------------------------
architecture Behavioral of Mux4x4_1x4_VHDL is
	Signal Sel:   std_logic_vector(1 downto 0); 
	Signal SelNr: integer range 0 to 3;
begin
	Sel   <= S1&S0;	            -- Sel(1:0) = S1&S0 (2-bit bus)
	SelNr	<= Conv_Integer( Sel);	-- Convert to integer.

	with SelNr select
   Y <= A when 0,		 				-- Y <= A when SelNr = 0
	     B when 1,		 				-- Y <= B when SelNr = 1
		  C when 2,		 				-- Y <= C when SelNr = 2
		  D when 3;		 				-- Y <= D when SelNr = 3

	Dp <= Dp0 when Sel="00" else	-- Dp <= Dp0 when SelNr = 0 
	      Dp1 when Sel="01" else	-- Dp <= Dp1 when SelNr = 1 
	      Dp2 when Sel="10" else	-- Dp <= Dp2 when SelNr = 2 
			Dp3;				 			-- Dp <= Dp3 (otherwise)

end Behavioral;
